// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar
 
#include <avr/io.h>
#include "lcd-4x40.h"
#include <util/delay.h>
 
uint8_t enablepin=0;		//deklaration ENAPLEPIN
 
// ----------------------------------------------------------------------------
// Funktionsname:	lcd_data(unsigned char temp1)
// Beschreibung:	sendet ein Datenbyte an das LCD
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2008-01-04
// nderungen:
// ----------------------------------------------------------------------------
void lcd_data(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_RS_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
 
   temp1 = temp1 >> 4;
   temp1 = temp1 & 0x0F;
   
   LCD_PORT_4 &= ~(1<<LCD_D4);
   LCD_PORT_5 &= ~(1<<LCD_D5);
   LCD_PORT_6 &= ~(1<<LCD_D6);
   LCD_PORT_7 &= ~(1<<LCD_D7);
 
   if(temp1 & 0x01) LCD_PORT_4 |=(1<<LCD_D4) ;// setzen
   if(temp1 & 0x02) LCD_PORT_5 |=(1<<LCD_D5) ;
   if(temp1 & 0x04) LCD_PORT_6 |=(1<<LCD_D6) ;
   if(temp1 & 0x08) LCD_PORT_7 |=(1<<LCD_D7) ;
   
   lcd_enable2();
 
   temp2 = temp2 & 0x0F;
   
   LCD_PORT_4 &= ~(1<<LCD_D4);
   LCD_PORT_5 &= ~(1<<LCD_D5);
   LCD_PORT_6 &= ~(1<<LCD_D6);
   LCD_PORT_7 &= ~(1<<LCD_D7); 
   
   if(temp2 & 0x01) LCD_PORT_4 |=(1<<LCD_D4) ;// setzen
   if(temp2 & 0x02) LCD_PORT_5 |=(1<<LCD_D5) ;
   if(temp2 & 0x04) LCD_PORT_6 |=(1<<LCD_D6) ;
   if(temp2 & 0x08) LCD_PORT_7 |=(1<<LCD_D7) ;
   
   lcd_enable2();
   
   _delay_us(40);    //40us
}
 
// ----------------------------------------------------------------------------
// Funktionsname:	lcd_command(unsigned char temp1)
// Beschreibung:	sendet einen Befehl
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2008-01-04
// nderungen:
// ----------------------------------------------------------------------------
void lcd_command(unsigned char temp1)
{
   unsigned char temp2 = temp1;
 
   LCD_RS_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
 
   temp1 = temp1 >> 4;              // oberes Nibble holen
   temp1 = temp1 & 0x0F;
   
   LCD_PORT_4 &= ~(1<<LCD_D4);
   LCD_PORT_5 &= ~(1<<LCD_D5);
   LCD_PORT_6 &= ~(1<<LCD_D6);
   LCD_PORT_7 &= ~(1<<LCD_D7);            // maskieren
   
   if(temp1 & 0x01) LCD_PORT_4 |=(1<<LCD_D4) ;// setzen
   if(temp1 & 0x02) LCD_PORT_5 |=(1<<LCD_D5) ;
   if(temp1 & 0x04) LCD_PORT_6 |=(1<<LCD_D6) ;
   if(temp1 & 0x08) LCD_PORT_7 |=(1<<LCD_D7) ;
   lcd_enable();
 
   temp2 = temp2 & 0x0F;
   
   LCD_PORT_4 &= ~(1<<LCD_D4);
   LCD_PORT_5 &= ~(1<<LCD_D5);
   LCD_PORT_6 &= ~(1<<LCD_D6);
   LCD_PORT_7 &= ~(1<<LCD_D7);				// unteres Nibble holen und maskieren
   
   if(temp2 & 0x01) LCD_PORT_4 |=(1<<LCD_D4) ;// setzen
   if(temp2 & 0x02) LCD_PORT_5 |=(1<<LCD_D5) ;
   if(temp2 & 0x04) LCD_PORT_6 |=(1<<LCD_D6) ;
   if(temp2 & 0x08) LCD_PORT_7 |=(1<<LCD_D7) ;
   lcd_enable();

   
   _delay_us(42);    //40us
}


// -----------------------------------------------------------------------------------------
// Funktionsname:	lcd_enable2(void)
// Beschreibung:	setzt den Enable2 Pin kurz gltig und taktet somit die Daten zum LCD
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2008-01-04
// nderungen:
// -----------------------------------------------------------------------------------------
void lcd_enable2(void)
{
   
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers einfgen
   if(enablepin==0)
   {
   // http://www.mikrocontroller.net/topic/81974#685882
   LCD_EN1_PORT |= (1<<LCD_EN1);
    _delay_us(40);    //40us                 // kurze Pause
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
   // http://www.mikrocontroller.net/topic/80900
   LCD_EN1_PORT &= ~(1<<LCD_EN1);
   }
   else
   {
   LCD_EN2_PORT |= (1<<LCD_EN2);
    _delay_us(40);    
	LCD_EN2_PORT &= ~(1<<LCD_EN2);
	}
}


// ------------------------------------------------------------------------------------------
// Funktionsname:	lcd_enable(void)
// Beschreibung:	setzt den Enable Pin kurz gltig und taktet somit die Daten zum LCD
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2006-11-26
// nderungen:
// -------------------------------------------------------------------------------------------
void lcd_enable(void)
{
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers einfgen
   
   // http://www.mikrocontroller.net/topic/81974#685882
   LCD_EN1_PORT |= (1<<LCD_EN1);
   LCD_EN2_PORT |= (1<<LCD_EN2);
 
   _delay_us(40);    //40us                   // kurze Pause
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
   // http://www.mikrocontroller.net/topic/80900
   LCD_EN1_PORT &= ~(1<<LCD_EN1);
   LCD_EN2_PORT &= ~(1<<LCD_EN2);
   
	
} 


// -----------------------------------------------------------------------------------------
// Funktionsname:	lcd_init(void)
// Beschreibung:	initialisert das Display     
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2006-11-26
// nderungen:
// -----------------------------------------------------------------------------------------

// Muss ganz am Anfang des Programms aufgerufen werden.
void lcd_init(void)
{
   // Ports auf Ausgang schalten
   LCD_DDR_4 |=(1<<LCD_D4);
   LCD_DDR_5 |=(1<<LCD_D5);
   LCD_DDR_6 |=(1<<LCD_D6);
   LCD_DDR_7 |=(1<<LCD_D7);
   
   LCD_EN1_DDR |= (1<<LCD_EN1);
   LCD_EN2_DDR |= (1<<LCD_EN2);
   LCD_RS_DDR  |= (1<<LCD_RS);
   // muss 3mal hintereinander gesendet werden zur Initialisierung
 
  _delay_ms(50);    //15ms
   
   LCD_PORT_4 |=  (1<<LCD_D4);	//8Bit Modus gesetzt
   LCD_PORT_5 |=  (1<<LCD_D5);
   LCD_PORT_6 &= ~(1<<LCD_D6);
   LCD_PORT_7 &= ~(1<<LCD_D7);  
  
   LCD_RS_PORT &= ~(1<<LCD_RS);      // RS auf 0
   lcd_enable();
 
   _delay_ms(5);    //lautet Datenblatt > wie 4,1ms
   lcd_enable();
 
   _delay_ms(1);	//lautet Datenblatt > wie 100us
   lcd_enable();	
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren
 
   
   
   LCD_PORT_4 &=  ~(1<<LCD_D4);
   LCD_PORT_5 |=   (1<<LCD_D5);
   LCD_PORT_6 &=  ~(1<<LCD_D6);
   LCD_PORT_7 &=  ~(1<<LCD_D7);  
   
   lcd_enable();
   _delay_ms(1);    //1ms
 
   // 4Bit / 2 Zeilen / 5x8
   lcd_command(0x28);
    
   // Display ein / Cursor aus / kein Blinken
   lcd_command(0x0C); 
 
   // inkrement / kein Scrollen
   lcd_command(0x06);
 
   lcd_clear();
}


// -------------------------------------------------------------------------------------------
// Funktionsname:	LCD_clear()
// Beschreibung:	lscht das Display
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2006-11-26
// nderungen:
// -------------------------------------------------------------------------------------------
void lcd_clear(void)
{
   lcd_command(CLEAR_DISPLAY);
   _delay_ms(5);
}

 
// -------------------------------------------------------------------------------------------
// Funktionsname:	lcd_home(void)
// Beschreibung:	Sendet den Befehl: Cursor Home
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2006-11-26
// nderungen:
// -------------------------------------------------------------------------------------------
void lcd_home(void)
{
   lcd_command(CURSOR_HOME);
   _delay_ms(5);    //5ms
}
 

// -------------------------------------------------------------------------------------------
// Funktionsname:	set_cursor(uint8_t x, uint8_t y)
// Beschreibung:	setzt den Cursor in Zeile y (1..4) Spalte x (0..15)
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2006-11-26
// nderungen:
// -------------------------------------------------------------------------------------------
void set_cursor(uint8_t x, uint8_t y)
{
  uint8_t tmp;       //eventuell =0 entfernen!!!!!!!!!!!!!!!!!!!
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; enablepin=0; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; enablepin=0; break;    // 2. Zeile
    case 3: tmp=0x80+0x00+x; enablepin=1; break;    // 3. Zeile
    case 4: tmp=0x80+0x40+x; enablepin=1; break;    // 4. Zeile
  }
  lcd_command(tmp);
}
 

// -------------------------------------------------------------------------------------------
// Funktionsname:	lcd_string(char *data)
// Beschreibung:	Schreibt einen String auf das LCD
// Parameter:		-
// Rckgabewert:	-
// Erstellt:		2006-11-26
// nderungen:
// -------------------------------------------------------------------------------------------
// 
 
void lcd_string(char *data)
{
    while(*data) {
        lcd_data(*data);
        data++;
    }
}
